@@ -24,7 +24,7 @@ package Class::Prototyped;
 use strict;
 use Carp();
 
-$Class::Prototyped::VERSION = '1.01';
+$Class::Prototyped::VERSION = '1.10';
 
 sub import {
 	while (my $symbol = shift) {
@@ -65,30 +65,57 @@ sub import {
 sub new {
 	my $class = shift;
 
-	my $self = {};
-	my $package =
-		Class::Prototyped::Mirror::PREFIX
-		. substr("$self", 7, -1);    # HASH($package)
+	Carp::croak("odd number of slot parameters to new\n") if scalar(@_) % 2;
 
-	Carp::croak("odd number of arguments to new\n") if scalar(@_) % 2;
-
-	tie %$self, $class->reflect->tiedInterfacePackage();
-	$class->newPackage($package, $self, @_);
+	$class->newCore('new', undef, @_);
 }
 
 sub newPackage {
 	my $class   = shift;
 	my $package = shift;
 
+	Carp::croak("odd number of slot parameters to newPackage\n") if scalar(@_) % 2;
+
+	$class->newCore('newPackage', $package, @_);
+}
+
+# Creates a copy of an object
+sub clone {
+	my $original = shift;
+
+	Carp::croak("odd number of slot parameters to clone\n") if scalar(@_) % 2;
+
+	$original->newCore('clone', undef, @_);
+}
+
+sub clonePackage {
+	my $original = shift;
+	my $package = shift;
+
+	Carp::croak("odd number of slot parameters to clonePackage\n") if scalar(@_) % 2;
+
+	$original->newCore('clonePackage', $package, @_);
+}
+
+
+sub newCore {
+	my $class   = shift;
+	my $caller  = shift;
+	my $package = shift;
+
+	my $isPackage = substr($caller, -7) eq 'Package';
+	my $isNew = substr($caller, 0, 3) eq 'new';
+	my $isClone = substr($caller, 0, 5) eq 'clone';
+
+	Carp::croak("odd number of slot parameters to $caller\n") if scalar(@_) % 2;
+
 	my $class_mirror = $class->reflect;
 
-	my ($self, $tied);
+	my($self, $tied);
 	{
 		no strict 'refs';
 
-		if (substr($package, 0, Class::Prototyped::Mirror::PREFIX_LENGTH) ne
-			Class::Prototyped::Mirror::PREFIX)
-		{
+		if ( $isPackage ) {
 			if (scalar(keys %{"$package\::"})) {
 				Carp::croak(
 					"attempt to use newPackage with already existing package\n"
@@ -100,7 +127,11 @@ sub newPackage {
 			$Class::Prototyped::Mirror::objects{$package} = $self = \%self;
 		}
 		else {
-			$self = shift;
+			$self = {};
+			$package = Class::Prototyped::Mirror::PREFIX . substr("$self", 7, -1);    # HASH($package)
+
+			tie %$self, $class->reflect->tiedInterfacePackage();
+
 			$tied = tied %$self;
 			*{"$package\::DESTROY"} = \&Class::Prototyped::DESTROY;
 		}
@@ -114,15 +145,20 @@ sub newPackage {
 
 	bless $self, $package;    # in my own package
 
-	$class = ref($class) || $class;    # allow object to provide a class
+	my $parsedSlots = scalar @_ || $isClone ?
+			$self->reflect->addSlotsParser( ( $isClone ? $class->reflect->getSlots() : () ), @_ ) :
+			[];
 
-	my $parsedSlots = scalar @_ ? $self->reflect->addSlotsParser(@_) : [];
+	if ( $isNew )	{
+		my $classstar = ( ref($class) &&
+				substr(ref($class), 0, Class::Prototyped::Mirror::PREFIX_LENGTH) ne 
+					Class::Prototyped::Mirror::PREFIX
+			) ? ref($class) : $class;   # allow object for named class to provide a class name
 
-	if (!(grep { $_->[0] eq 'class*' } @$parsedSlots) &&
-			substr($class, 0, Class::Prototyped::Mirror::PREFIX_LENGTH) ne
-			Class::Prototyped::Mirror::PREFIX && $class ne 'Class::Prototyped')
-	{
-		unshift(@$parsedSlots, @{$self->reflect->addSlotsParser('class*' => $class)});
+		if (!(grep { $_->[0] eq 'class*' } @$parsedSlots) && $classstar ne 'Class::Prototyped')
+		{
+			unshift(@$parsedSlots, @{$self->reflect->addSlotsParser('class*' => $classstar)});
+		}
 	}
 
 	$self->reflect->addParsedSlots($parsedSlots) if scalar(@$parsedSlots);
@@ -130,14 +166,6 @@ sub newPackage {
 	return $self;
 }
 
-# Creates a copy of an object
-sub clone {
-	my $original = shift;
-
-	(ref($original) ? $original : 'Class::Prototyped')
-		->new($original->reflect->getSlots, @_);
-}
-
 sub reflect {
 	return $Class::Prototyped::Mirror::mirrors{ $_[0] } || Class::Prototyped::Mirror->new($_[0]);
 }
@@ -435,10 +463,10 @@ sub new {
 		if (substr(ref($entity), 0, Class::Prototyped::Mirror::PREFIX_LENGTH) eq
 				Class::Prototyped::Mirror::PREFIX)
 		{
-			return bless \$entity, $class;
+			return bless \$entity, 'Class::Prototyped::Mirror';
 		}
 		elsif ($Class::Prototyped::Mirror::objects{ ref($entity) } == $entity) {
-			return $Class::Prototyped::Mirror::mirrors{ $entity } ||= bless \$entity, $class;
+			return $Class::Prototyped::Mirror::mirrors{ $entity } ||= bless \$entity, 'Class::Prototyped::Mirror';
 		}
 		else {
 			return Class::Prototyped::Mirror::Normal->new($entity);
@@ -475,7 +503,7 @@ sub new {
 
 		bless $object, $entity;
 	}
-	return $Class::Prototyped::Mirror::mirrors{ $entity } ||= bless \$object, $class;
+	return $Class::Prototyped::Mirror::mirrors{ $entity } ||= bless \$object, 'Class::Prototyped::Mirror';
 }
 
 
@@ -490,8 +518,9 @@ sub new {
 	my $class = shift;
 	my($entity) = @_;
 
-	my $temp = $class->SUPER::new(ref($entity));
-	my $self = bless \${ $temp }, $class;
+	my $temp = Class::Prototyped::Mirror->new(ref($entity));
+
+	my $self = bless \(my $o = ${$temp}), $class;
 	$Class::Prototyped::Mirror::Normal::superselfs->{$self} = $entity;
 	return $self;
 }
@@ -1872,7 +1901,7 @@ Just as you can C<clone> objects, you can C<clone> classes that are derived from
 C<Class::Prototyped>. This creates a new object that has a copy of all of the 
 slots that were defined in the class.  Note that if you simply want to be able 
 to use C<Data::Dumper> on a class, calling C<< MyClass->reflect->object >> is 
-the preferred approach.  Or simply use the C<dump> mirror method.
+the preferred approach.  Even easier would be to use the C<dump> mirror method.
 
 The code that implements reflection on classes automatically creates slot
 names for package methods as well as parent slots for the entries in C<@ISA>.
@@ -1973,18 +2002,17 @@ code does what is expected:
 
 =head2 Object Class
 
-The special parent slot C<class*> is used to indicate object class.  When you
-create C<Class::Prototyped> objects, the C<class*> slot is B<not> set.  If,
-however, you create objects by calling C<new> on a class that inherits from
-C<Class::Prototyped>, the slot C<class*> points to the package name.
+The special parent slot C<class*> is used to indicate object class.  When you 
+create C<Class::Prototyped> objects by calling C<< Class::Prototyped->new() >>, 
+the C<class*> slot is B<not> set.  If, however, you create objects by calling 
+C<new> on a class or object that inherits from C<Class::Prototyped>, the slot 
+C<class*> points to the package name if C<new> was called on a named class, or 
+the object if C<new> was called on an object.
 
 The value of this slot can be returned quite easily like so:
 
     $foo->reflect->class;
 
-Class is set when C<new> is called on a package or object that has a named
-package.
-
 
 =head2 Calling Inherited Methods
 
@@ -2203,13 +2231,22 @@ doing this.
 
 =head2 new() - Construct a new C<Class::Prototyped> object.
 
-A new object is created.  If this is called on a class that inherits from
-C<Class::Prototyped>, and C<class*> is not being passed as a slot in the
-argument list, the slot C<class*> will be the first element in the inheritance
+A new object is created.  If this is called on a class or object that inherits 
+from C<Class::Prototyped>, and C<class*> is not being passed as a slot in the 
+argument list, the slot C<class*> will be the first element in the inheritance 
 list.
 
+When called on named classes, either via the package name or via the object 
+(i.e. C<< MyPackage->reflect->object() >>), C<class*> is set to the package 
+name.  When called on an object, C<class*> is set to the object on which C<new> 
+was called.
+
 The passed arguments are handed off to C<addSlots>.
 
+Note that C<new> calls C<newCore>, so if you want to override C<new>, but want 
+to ensure that your changes are applicable to C<newPackage>, C<clone>, and 
+C<clonePackage>, you may wish to override C<newCore>.
+
 For instance, the following will define a new C<Class::Prototyped> object with
 two method slots and one field slot:
 
@@ -2228,6 +2265,14 @@ before C<class*>, which points to C<MyClass>):
         [qw(bar* promote)] => $bar,
     );
 
+The following will create a new object that inherits behavior from C<$bar> with 
+one field slot, C<field1>, and one parent slot, C<class*>, that points to 
+C<$bar>.
+
+    my $foo = $bar->new(
+        field1  => 123,
+    );
+
 If you want to create normal Perl objects as child objects of a 
 C<Class::Prototyped> class in order to improve performance, implement your own 
 standard Perl C<new> method:
@@ -2250,23 +2295,53 @@ inspecting the class to which the object belongs.
 =head2 newPackage() - Construct a new C<Class::Prototyped> object in a
 specific package.
 
-Just like C<new>, but instead of creating the new object with an arbitrary
-package name (actually, not entirely arbitrary - it's generally based on the
-hash memory address), the first argument is used as the name of the package.
+Just like C<new>, but instead of creating the new object with an arbitrary 
+package name (actually, not entirely arbitrary - it's generally based on the 
+hash memory address), the first argument is used as the name of the package.  
+This creates a named class.  The same behavioral rules for C<class*> described 
+above for C<new> apply to C<newPackage> (in fact, C<new> calls C<newPackage>).
 
 If the package name is already in use, this method will croak.
 
 =head2 clone() - Duplicate me
 
-Duplicates an existing object or class. and allows you to add or override
+Duplicates an existing object or class and allows you to add or override
 slots. The slot definition is the same as in B<new()>.
 
   my $p2 = $p1->clone(
       sub1 => sub { print "this is sub1 in p2" },
   );
 
-It calls C<new> on the object to create the new object, so if C<new> has been
-overriden, the overriden C<new> will be called.
+It calls C<newCore> to create the new object*, so if you have overriden C<new>, 
+you should contemplate overriding C<clone> in order to ensure that behavioral 
+changes made to C<new> that would be applicable to C<clone> are implemented.  Or 
+simply override C<newCore>.
+
+=head2 clonePackage()
+
+Just like C<clone>, but instead of creating the new object with an arbitrary 
+package name (actually, not entirely arbitrary - it's generally based on the 
+hash memory address), the first argument is used as the name of the package.  
+This creates a named class.
+
+If the package name is already in use, this method will croak.
+
+=head2 newCore()
+
+This implements the core functionality involved in creating a new object.  The 
+first passed parameter will be the name of the caller - either C<new>, 
+C<newPackage>, C<clone>, or C<clonePackage>.  The second parameter is the name 
+of the package if applicable (i.e. for C<newPackage> and C<clonePackage>) calls, 
+C<undef> if inapplicable.  The remainder of the parameters are any slots to be 
+added to the newly created object/package.
+
+If called with C<new> or C<newPackage>, the C<class*> slot will be prepended to 
+the slot list if applicable.  If called with C<clone> or C<clonePackage>, all 
+slots on the receiver will be prepended to the slot list.
+
+If you wish to add behavior to object instantiation that needs to be present in 
+all four of the instantiators (i.e. instance tracking), it may make sense to 
+override C<newCore> so that you implement the code in only one place.
 
 =head2 reflect() - Return a mirror for the object or class
 
@@ -2787,8 +2862,7 @@ C<thisObject()>.
 
 =head1 AUTHOR
 
-Written by Ned Konz, perl@bike-nomad.com and Toby Ovod-Everett, 
-toby@ovod-everett.org. 5.005_03 porting by chromatic.
+Written by Ned Konz, perl@bike-nomad.com and Toby Ovod-Everett, toby@ovod-everett.org. 5.005_03 porting by chromatic.
 
 Toby Ovod-Everett is currently maintaining the package.
 
